@@ -402,15 +402,15 @@ class Actuator:  # 3EF0, 3EF1
     ACTUATOR_CYCLE = "actuator_cycle"
     ACTUATOR_ENABLED = "actuator_enabled"  # boolean
     ACTUATOR_STATE = "actuator_state"
-    ENABLED = "enabled"
-    MODULATION_LEVEL = "modulation_level"  # percentge (0.0-1.0)
+    MODULATION_LEVEL = "modulation_level"  # percentage (0.0-1.0)
 
     @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
         if discover_flag & DISCOVER_STATUS and not self._faked:
-            self._send_cmd(_3EF1)  # No RPs to 3EF0
+            # NOTE: No need to send periodic RQ/3EF1s to an OTB, use RQ/3220/11s
+            self._send_cmd(_3EF1)  # NOTE: No RPs to RQ/3EF0
 
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
@@ -426,22 +426,12 @@ class Actuator:  # 3EF0, 3EF1
     def actuator_state(self) -> Optional[dict]:  # 3EF0
         return self._msg_value(_3EF0)
 
-    @property
-    def enabled(self) -> Optional[bool]:  # 3EF0, 3EF1
-        """Return the actuator's current state."""
-        return self._msg_value((_3EF0, _3EF1), key=self.ACTUATOR_ENABLED)
-
-    @property
-    def modulation_level(self) -> Optional[float]:  # 3EF0/3EF1
-        return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
-
     @property
     def status(self) -> dict:
         return {
             **super().status,
             self.ACTUATOR_CYCLE: self.actuator_cycle,
             self.ACTUATOR_STATE: self.actuator_state,
-            self.MODULATION_LEVEL: self.modulation_level,  # TODO: keep? (is duplicate)
         }
 
 
@@ -687,11 +677,8 @@ class Temperature(Fakeable):  # 30C9 (fakeable)
         if not self._faked:
             raise RuntimeError(f"Can't set value for {self} (Faking is not enabled)")
 
-        cmd = Command.put_sensor_temp(self.id, value)
-        # cmd = Command.put_sensor_temp(
-        #     self._gwy.hgi.id if self == self._gwy.hgi._faked_thm else self.id, value
-        # )
-        self._gwy.send_cmd(cmd)
+        self._gwy.send_cmd(Command.put_sensor_temp(self.id, value))
+        # lf._gwy.send_cmd(Command.get_zone_temp(self._ctl.id, self.zone.idx))
 
     @property
     def status(self) -> dict:
@@ -716,7 +703,7 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
         super()._discover(discover_flag=discover_flag)
 
         if discover_flag & DISCOVER_STATUS and not self._faked:
-            self._send_cmd(_0008)
+            self._send_cmd(_0008)  # NOTE: No RPs to RQ/0009
 
     def _handle_msg(self, msg) -> None:
         if msg.src.id == self.id:
@@ -742,7 +729,7 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
             qos = {"priority": Priority.HIGH, "retries": 3}
             [self._gwy.send_cmd(cmd, **qos) for _ in range(1)]
 
-        elif msg.code == _0009 and msg.verb == I_:
+        elif msg.code == _0009:  # can only be I, from a controller
             pass
 
         elif msg.code == _3B00 and msg.verb == I_:
@@ -806,6 +793,27 @@ class HGInterface(DeviceBase):  # HGI (18:), was GWY
         self._faked_ext = None
         self._faked_thm = None
 
+        # self. _proc_schema(**kwargs)
+
+    def _set_ctl(self, ctl) -> None:  # self._ctl
+        """Set the device's parent controller, after validating it."""
+        _LOGGER.debug("%s: can't (really) have a controller %s", self)
+
+    def _proc_schema(self, schema) -> None:
+        if schema.get("fake_bdr"):
+            self._faked_bdr = self._gwy._get_device(self.id, class_="BDR", faked=True)
+
+        if schema.get("fake_ext"):
+            self._fake_ext = self._gwy._get_device(self.id, class_="BDR", faked=True)
+
+        if schema.get("fake_thm"):
+            self._fake_thm = self._gwy._get_device(self.id, class_="BDR", faked=True)
+
+    @discovery_filter
+    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+        # of no value for a HGI80-compatible device
+        return
+
     def _handle_msg(self, msg) -> None:
         def fake_addrs(msg, faked_dev):
             msg.src == faked_dev if msg.src is self else self
@@ -1093,6 +1101,12 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
     def __repr__(self) -> str:
         return f"{self.id} ({self._domain_id}): {self.temperature}"
 
+    def _handle_msg(self, msg) -> None:
+        super()._handle_msg(msg)
+
+        if msg.code == _1260:
+            self._gwy.send_cmd(Command.get_dhw_temp(self._ctl.id))
+
     @property
     def dhw_params(self) -> Optional[dict]:  # 10A0
         return self._msg_value(_10A0)
@@ -1256,30 +1270,35 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
             return
 
     @property
-    def boiler_water_temperature(self) -> Optional[float]:  # 3220/0x19
+    def modulation_level(self) -> Optional[float]:  # 3EF0/3EF1
+        # TODO: for OTB, deprecate in favour of RP/3220/11
+        return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
+
+    @property
+    def boiler_water_temperature(self) -> Optional[float]:  # 3220/19
         return self._ot_msg_value(0x19)
 
     @property
-    def ch_water_pressure(self) -> Optional[float]:  # 3220/0x12
+    def ch_water_pressure(self) -> Optional[float]:  # 3220/12
         return self._ot_msg_value(0x12)
 
     @property
-    def dhw_flow_rate(self) -> Optional[float]:  # 3220/0x13
+    def dhw_flow_rate(self) -> Optional[float]:  # 3220/13
         return self._ot_msg_value(0x13)
 
     @property
-    def dhw_temperature(self) -> Optional[float]:  # 3220/0x1A
+    def dhw_temperature(self) -> Optional[float]:  # 3220/1A
         return self._ot_msg_value(0x1A)
 
-    @property  # HA
-    def relative_modulation_level(self) -> Optional[float]:  # 3220/0x11
+    @property
+    def relative_modulation_level(self) -> Optional[float]:  # 3220/11
         return self._ot_msg_value(0x11)
 
     @property
-    def return_water_temperature(self) -> Optional[float]:  # 3220/0x1C
+    def return_water_temperature(self) -> Optional[float]:  # 3220/1C
         return self._ot_msg_value(0x1C)
 
-    @property  # HA
+    @property
     def boiler_setpoint(self) -> Optional[float]:  # 22D9
         return self._msg_value(_22D9, key=self.BOILER_SETPOINT)
 
@@ -1374,6 +1393,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
             **super().status,
             self.BOILER_SETPOINT: self.boiler_setpoint,
             self.OPENTHERM_STATUS: self.opentherm_status,
+            self.MODULATION_LEVEL: self.modulation_level,
         }
 
 
@@ -1445,6 +1465,7 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
     _class = DEVICE_CLASS.BDR
     _types = ("13",)
 
+    ACTIVE = "active"
     TPI_PARAMS = "tpi_params"
     # _STATE = super().ENABLED, or relay_demand
 
@@ -1474,11 +1495,7 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
 
         super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_3B00)  # TODO: will RP a RQ/3B00 if is a heater_relay?
-
         if discover_flag & DISCOVER_PARAMS and not self._faked:
-            self._send_cmd(_0009)
             self._send_cmd(_1100)
 
     def _handle_msg(self, msg) -> None:
@@ -1492,6 +1509,12 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
         #     # for code in (_0008, _3EF1):
         #     #     self._send_cmd(code, delay=1)
 
+    @property
+    def active(self) -> Optional[bool]:  # 3EF0, 3EF1
+        """Return the actuator's current state."""
+        result = self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
+        return None if result is None else bool(result)
+
     @property
     def role(self) -> Optional[str]:
         """Return the role of the BDR91A (there are six possibilities)."""
@@ -1525,6 +1548,13 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
             self.TPI_PARAMS: self.tpi_params,
         }
 
+    @property
+    def status(self) -> dict:
+        return {
+            **super().status,
+            self.ACTIVE: self.active,
+        }
+
 
 class TrvActuator(BatteryState, HeatDemand, Setpoint, Temperature, Device):  # TRV (04):
     """The TRV class, such as a HR92."""
