@@ -7,12 +7,15 @@ Decode/process a message (payload into JSON).
 """
 
 import logging
+import re
 from datetime import timedelta as td
+from functools import lru_cache
 from typing import Any, Optional, Tuple, Union
 
 from .address import Address, dev_id_to_str
-from .parsers import parse_payload
-from .ramses import CODE_IDX_COMPLEX, RAMSES_CODES, RQ_NO_PAYLOAD
+from .exceptions import InvalidPacketError, InvalidPayloadError
+from .parsers import PAYLOAD_PARSERS, parser_unknown
+from .ramses import CODE_IDX_COMPLEX, CODE_RQ_COMPLEX, RAMSES_CODES, RAMSES_DEVICES
 
 from .const import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
 from .const import (  # noqa: F401, isort: skip
@@ -99,7 +102,10 @@ class Message:
     IS_EXPIRING = 0.8  # expected lifetime == 1.0
 
     def __init__(self, gwy, pkt) -> None:
-        """Create a message from a valid packet."""
+        """Create a message from a valid packet.
+
+        Will raise InvalidPacketError if it is invalid.
+        """
         self._gwy = gwy
         self._pkt = pkt
 
@@ -109,8 +115,6 @@ class Message:
         self._addrs = pkt.addrs
 
         self.dtm = pkt.dtm
-        self._date = pkt._date
-        self._time = pkt._time
 
         self.verb = pkt.verb
         self.seqn = pkt.seqn
@@ -119,18 +123,13 @@ class Message:
         self.raw_payload = pkt.payload
 
         self.code_name = CODE_NAMES.get(self.code, f"unknown_{self.code}")
-        self._payload = None
+
+        self._payload = self._validate(self.raw_payload)  # ? raise InvalidPacketError
+
         self._str = None
-
-        self.__has_payload = None
-
-        self.__expired = None
+        self._expired_ = None
         self._is_fragment = None
 
-        self._is_valid = None
-        if not self.is_valid:
-            raise ValueError(f"Invalid message: {pkt}")
-
     def __repr__(self) -> str:
         """Return an unambiguous string representation of this object."""
         return repr(self._pkt)  # or str?
@@ -182,36 +181,24 @@ class Message:
             return NotImplemented
         return self.dtm < other.dtm
 
-    @property
-    def _has_payload(self) -> bool:
-        """Return False if no payload (may falsely Return True).
-
-        The message (i.e. the raw payload) may still have an idx.
-        """
-
-        if self.__has_payload is not None:
-            return self.__has_payload
-
-        self.__has_payload = not bool(
-            self.len == 1
-            # or (self.len == 2 and self.verb == RQ)  # NOTE: see 0016
-            or (self.verb == RQ and self.code in RQ_NO_PAYLOAD)
-        )
-
-        return self.__has_payload
-
     @property
     def payload(self) -> Any:  # Any[dict, List[dict]]:
         """Return the payload."""
         return self._payload
 
     @property
-    def _has_array(self) -> bool:
-        """Return True if the message's raw payload is an array.
+    def _has_payload(self) -> bool:
+        """Return False if there is no payload (may falsely Return True).
 
-        Does not neccessarily require a valid payload.
+        The message (i.e. the raw payload) may still have an idx.
         """
 
+        return self._pkt._has_payload
+
+    @property
+    def _has_array(self) -> bool:
+        """Return True if the message's raw payload is an array."""
+
         return self._pkt._has_array
 
     @property
@@ -287,25 +274,25 @@ class Message:
     def _expired(self) -> Tuple[bool, Optional[bool]]:
         """Return True if the message is dated (does not require a valid payload)."""
 
-        if self.__expired is not None:
-            if self.__expired == self.CANT_EXPIRE:
+        if self._expired_ is not None:
+            if self._expired_ == self.CANT_EXPIRE:
                 return False
-            if self.__expired >= self.HAS_EXPIRED * 2:  # TODO: should delete?
+            if self._expired_ >= self.HAS_EXPIRED * 2:  # TODO: should delete?
                 return True
 
         if self.code == _1F09 and self.verb == I_:
             # RQs won't have remaining_seconds
             # RP/Ws will have only partial cycle times
             timeout = td(seconds=self.payload["remaining_seconds"])
-            self.__expired = (self._gwy._dt_now() - self.dtm) / timeout
+            self._expired_ = (self._gwy._dt_now() - self.dtm) / timeout
         else:
-            self.__expired = self._pkt._expired
+            self._expired_ = self._pkt._expired
 
-        if self.__expired is False:  # treat as never expiring
+        if self._expired_ is False:  # treat as never expiring
             _LOGGER.info("%s # cant expire", self._pkt)
-            self.__expired = self.CANT_EXPIRE
+            self._expired_ = self.CANT_EXPIRE
 
-        elif self.__expired >= self.HAS_EXPIRED:  # TODO: should renew?
+        elif self._expired_ >= self.HAS_EXPIRED:  # TODO: should renew?
             if any(
                 (
                     self.code == _1F09 and self.verb != I_,
@@ -315,13 +302,13 @@ class Message:
                 _logger = _LOGGER.info
             else:
                 _logger = _LOGGER.warning
-            _logger("%s # has expired %s", self._pkt, f"({self.__expired * 100:1.0f}%)")
+            _logger("%s # has expired %s", self._pkt, f"({self._expired_ * 100:1.0f}%)")
 
-        # elif self.__expired >= self.IS_EXPIRING:  # this could log multiple times
+        # elif self._expired_ >= self.IS_EXPIRING:  # this could log multiple times
         #     _LOGGER.error("%s # is expiring", self._pkt)
 
         # and self.dtm >= self._gwy._dt_now() - td(days=7)  # TODO: should be none >7d?
-        return self.__expired >= self.HAS_EXPIRED
+        return self._expired_ >= self.HAS_EXPIRED
 
     @property
     def _is_fragment_WIP(self) -> bool:
@@ -343,12 +330,146 @@ class Message:
 
         return self._is_fragment
 
-    @property
-    def is_valid(self) -> bool:  # Main code here
-        """Parse the payload, return True if the message payload is valid."""
+    def _validate(self, raw_payload) -> dict:
+        """Validate the message, and parse the payload if so.
 
-        if self._is_valid is None:
-            self._payload = parse_payload(self, logger=_LOGGER)
-            # self._payload = {k: v for k, v in self._raw_payload.items() if k[:1] != "_"}
-            self._is_valid = self._payload is not None
-        return self._is_valid
+        Raise an exception if it is not valid.
+        """
+
+        # # STEP 0: Check verb/code pair against src/dst device type & payload
+        # if self.code not in RAMSES_CODES:
+        #     raise InvalidPacketError(f"Unknown code: {self.code}")
+
+        _check_verb_code_src(self)  # ? raise InvalidPacketError
+        _check_verb_code_dst(self)  # ? raise InvalidPacketError
+        _check_verb_code_payload(self, self.raw_payload)  # ? raise InvalidPayloadError
+
+        if not self._has_payload or (
+            self.verb == RQ and self.code not in CODE_RQ_COMPLEX
+        ):
+            # _LOGGER.error("%s", msg)
+            return {}
+
+        try:  # parse the payload
+            result = PAYLOAD_PARSERS.get(self.code, parser_unknown)(
+                self.raw_payload, self
+            )
+            assert isinstance(
+                result, (dict, list)
+            ), f"invalid payload type: {type(result)}"
+
+        except AssertionError as err:
+            # beware: HGI80 can send parseable but 'odd' packets +/- get invalid reply
+            (
+                _LOGGER.exception
+                if DEV_MODE and self.src.type != "18"
+                else _LOGGER.exception
+            )("%s << %s", self._pkt, f"{err.__class__.__name__}({err})")
+
+        except (InvalidPacketError, InvalidPayloadError) as err:  # CorruptEvohomeError
+            (_LOGGER.exception if DEV_MODE else _LOGGER.warning)(
+                "%s << %s", self._pkt, err
+            )
+
+        except (AttributeError, LookupError, TypeError, ValueError) as err:  # TODO: dev
+            _LOGGER.exception(
+                "%s << Coding error: %s", self._pkt, f"{err.__class__.__name__}({err})"
+            )
+
+        except NotImplementedError:  # parser_unknown (unknown packet code)
+            _LOGGER.warning("%s << Unknown packet code (cannot parse)", self._pkt)
+
+        # _LOGGER.error("%s", msg)
+        return result if isinstance(result, list) else {**self._idx, **result}
+
+
+@lru_cache(maxsize=256)
+def re_compile_re_match(regex, string) -> bool:
+    return re.compile(regex).match(string)
+
+
+def _check_verb_code_src(msg) -> None:
+    """Validate the packet's verb/code pair against its source device type.
+
+    Raise InvalidPacketError if the meta data is invalid, otherwise simply return.
+    """
+    if msg.src.type not in RAMSES_DEVICES:
+        raise InvalidPacketError(f"Unknown src device type: {msg.src.id}")
+
+    if msg.src.type == "18":  # TODO: make a dynamic list if sensor/relay faking
+        if msg.code not in RAMSES_CODES:  # NOTE: HGI can send whatever it likes
+            raise InvalidPacketError(f"Unknown code for {msg.src.id} to Tx: {msg.code}")
+        return
+
+    if msg.code not in RAMSES_DEVICES[msg.src.type]:
+        raise InvalidPacketError(f"Invalid code for {msg.src.id} to Tx: {msg.code}")
+
+    if msg.verb not in RAMSES_DEVICES[msg.src.type][msg.code]:
+        raise InvalidPacketError(
+            f"Invalid verb/code for {msg.src.id} to Tx: {msg.verb}/{msg.code}"
+        )
+
+
+def _check_verb_code_dst(msg) -> None:
+    """Validate the packet's verb/code pair against its destination device type.
+
+    Raise InvalidPacketError if the meta data is invalid, otherwise simply return.
+    """
+
+    # check that the destination would normally respond...
+    if "18" in (msg.src.type, msg.dst.type) or msg.dst.type in ("--", "63"):
+        return  # could omit this check to enforce strict checking
+
+    if msg.dst.type not in RAMSES_DEVICES:
+        raise InvalidPacketError(f"Unknown dst device type: {msg.dst.id}")
+
+    if msg.verb == I_:  # receiving an I is not in the schema, so cant be tested
+        return
+
+    # HACK: these exceptions need sorting
+    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"01/{RQ}/{_3EF1}",):
+        return
+
+    if msg.code not in RAMSES_DEVICES[msg.dst.type]:  # NOTE: is not OK for Rx
+        #  I --- 04:253797 --:------ 01:063844 1060 003 056401
+        # HACK: these exceptions need sorting
+        # if msg.code in (_1060, ):
+        #     return
+        raise InvalidPacketError(f"Invalid code for {msg.dst.id} to Rx: {msg.code}")
+
+    # HACK: these exceptions need sorting
+    if f"{msg.verb}/{msg.code}" in (f"{W_}/{_0001}",):
+        return
+
+    # HACK: these exceptions need sorting
+    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"13/{RQ}/{_3EF0}",):
+        return
+
+    verb = {RQ: RP, RP: RQ, W_: I_}[msg.verb]
+    if verb not in RAMSES_DEVICES[msg.dst.type][msg.code]:
+        raise InvalidPacketError(
+            f"Invalid verb/code for {msg.dst.id} to Rx: {msg.verb}/{msg.code}"
+        )
+
+
+def _check_verb_code_payload(msg, payload) -> None:
+    """Validate the packet's verb/code pair against its payload.
+
+    Raise an InvalidPayloadError if the payload is invalid, otherwise simply return.
+    Some parsers may also raise InvalidPayloadError (e.g. 3220).
+    """
+
+    try:
+        regex = RAMSES_CODES[msg.code][msg.verb]
+        if not re_compile_re_match(regex, payload):
+            raise InvalidPayloadError(f"{msg._pkt} < Payload doesn't match '{regex}'")
+    except KeyError:
+        pass  # TODO: raise - missing entry in RAMSES schema
+
+    # TODO: put this back, or leave it to the parser?
+    # if msg.code == _3220:
+    #     msg_id = int(msg.raw_payload[4:6], 16)
+    #     if msg_id not in OPENTHERM_MESSAGES:  # parser uses OTB_MSG_IDS
+    #         raise InvalidPayloadError(
+    #             f"OpenTherm: Unsupported data-id: 0x{msg_id:02X} ({msg_id})"
+    #         )
