@@ -9,11 +9,9 @@
 # - ReneKlootwijk: 3EF0
 
 import logging
-import re
 from datetime import datetime as dt
 from datetime import timedelta as td
-from functools import lru_cache
-from typing import Optional, Union
+from typing import Any, Optional, Union
 
 from .const import (
     _000C_DEVICE_TYPE,
@@ -27,7 +25,7 @@ from .const import (
     SYSTEM_MODE,
     ZONE_MODE,
 )
-from .exceptions import InvalidPacketError, InvalidPayloadError
+from .exceptions import InvalidPayloadError
 from .helpers import (
     bool_from_hex,
     date_from_hex,
@@ -42,7 +40,6 @@ from .helpers import (
     valve_demand,
 )
 from .opentherm import EN, MSG_DESC, MSG_ID, MSG_NAME, MSG_TYPE, decode_frame
-from .ramses import CODE_RQ_COMPLEX, RAMSES_CODES, RAMSES_DEVICES
 from .version import VERSION
 
 from .const import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
@@ -106,8 +103,6 @@ from .const import (  # noqa: F401, isort: skip
     _PUZZ,
 )
 
-__all__ = ("parse_payload",)
-
 DEV_MODE = __dev_mode__ and False
 TEST_MODE = False  # enable to test constructors (usu. W)
 
@@ -116,112 +111,9 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-@lru_cache(maxsize=256)
-def re_compile_re_match(regex, string) -> bool:
-    return re.compile(regex).match(string)
-
-
-def _check_verb_code_src(msg) -> None:
-    """Check the packet's source device type against its verb/code pair."""
-    if msg.src.type not in RAMSES_DEVICES:
-        raise InvalidPacketError(f"Unknown src device type: {msg.src.id}")
-
-    if msg.src.type == "18":  # TODO: make a dynamic list if sensor/relay faking
-        if msg.code not in RAMSES_CODES:  # NOTE: HGI can send whatever it likes
-            # currently, there isn't complete detail in RAMSES_DEVICES for HGI
-            raise InvalidPacketError(f"Unknown code for {msg.src.id} to Tx: {msg.code}")
-        return
-
-    if msg.code not in RAMSES_DEVICES[msg.src.type]:
-        raise InvalidPacketError(f"Invalid code for {msg.src.id} to Tx: {msg.code}")
-
-    if msg.verb not in RAMSES_DEVICES[msg.src.type][msg.code]:
-        raise InvalidPacketError(
-            f"Invalid verb/code for {msg.src.id} to Tx: {msg.verb}/{msg.code}"
-        )
-
-
-def _check_verb_code_dst(msg) -> None:
-    """Check the packet's destination device type against its verb/code pair."""
-
-    # check that the destination would normally respond...
-    if "18" in (msg.src.type, msg.dst.type) or msg.dst.type in ("--", "63"):
-        # could leave this out to enforce strict checking
-        return
-
-    if msg.dst.type not in RAMSES_DEVICES:
-        raise InvalidPacketError(f"Unknown dst device type: {msg.dst.id}")
-
-    if msg.verb == I_:
-        return
-
-    # HACK: these exceptions need sorting
-    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"01/{RQ}/{_3EF1}",):
-        return
-
-    if msg.code not in RAMSES_DEVICES[msg.dst.type]:  # NOTE: is not OK for Rx
-        #  I --- 04:253797 --:------ 01:063844 1060 003 056401
-        # HACK: these exceptions need sorting
-        # if msg.code in (_1060, ):
-        #     return
-        raise InvalidPacketError(f"Invalid code for {msg.dst.id} to Rx: {msg.code}")
-
-    # HACK: these exceptions need sorting
-    if f"{msg.verb}/{msg.code}" in (f"{W_}/{_0001}",):
-        return
-
-    # HACK: these exceptions need sorting
-    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"13/{RQ}/{_3EF0}",):
-        return
-
-    verb = {RQ: RP, RP: RQ, W_: I_}[msg.verb]
-    if verb not in RAMSES_DEVICES[msg.dst.type][msg.code]:
-        raise InvalidPacketError(
-            f"Invalid verb/code for {msg.dst.id} to Rx: {msg.verb}/{msg.code}"
-        )
-
-
-def _check_verb_code_payload(msg, payload) -> None:
-    """Check the packet's payload against its verb/code pair."""
-
-    try:
-        regex = RAMSES_CODES[msg.code][msg.verb]
-        if not re_compile_re_match(regex, payload):
-            raise InvalidPayloadError(f"Payload doesn't match '{regex}'")
-    except KeyError:
-        pass  # TODO: raise
-
-    # TODO: put this back, or leave it to the parser?
-    # if msg.code == _3220:
-    #     msg_id = int(msg.raw_payload[4:6], 16)
-    #     if msg_id not in OPENTHERM_MESSAGES:  # parser uses OTB_MSG_IDS
-    #         raise InvalidPayloadError(
-    #             f"OpenTherm: Unsupported data-id: 0x{msg_id:02X} ({msg_id})"
-    #         )
-
-
 def parser_decorator(func):
-    """Validate message payload (or meta-data), e.g payload length)."""
-
-    def wrapper(*args, **kwargs) -> Optional[dict]:
-        """Check the length of a payload."""
-        payload, msg = args[0], args[1]
-
-        # STEP 0: Check verb/code pair against src/dst device type & payload
-        if msg.code not in RAMSES_CODES:
-            raise InvalidPacketError(f"Unknown code: {msg.code}")
-
-        _check_verb_code_src(msg)
-        _check_verb_code_dst(msg)
-
-        _check_verb_code_payload(msg, payload)  # must use payload, not msg.payload
-
-        if msg._has_payload or msg.verb == RQ and msg.code in CODE_RQ_COMPLEX:
-            result = func(*args, **kwargs)
-        else:
-            result = {}
-
-        return result if isinstance(result, list) else {**msg._idx, **result}
+    def wrapper(*args, **kwargs) -> Optional[Any]:
+        return func(*args, **kwargs)
 
     return wrapper
 
@@ -314,7 +206,7 @@ def parser_0005(payload, msg) -> Optional[dict]:  # TODO: needs a cleanup
     def _parser(seqx) -> dict:
         return {
             "_zone_type": seqx[2:4],
-            "zone_mask": (flag8(seqx[4:6]) + flag8(seqx[6:8])),
+            "zone_mask": (flag8(seqx[4:6], lsb=True) + flag8(seqx[6:8], lsb=True)),
             "zone_type": _0005_ZONE_TYPE.get(seqx[2:4], f"unknown_{seqx[2:4]}"),
         }
 
@@ -1750,17 +1642,40 @@ def parser_3ef0(payload, msg) -> dict:
             "blob": payload[8:],
         }
 
+    assert msg.len in (3, 6, 9)
     assert payload[:2] == "00", f"byte 1: {payload[:2]}"
 
-    if 1 < msg.len <= 3:
-        assert payload[2:4] in ("00", "C8", "FF"), f"byte 1: {payload[2:4]}"
+    if msg.len == 3:  # I|BDR|003
+        # .I --- 13:042805 --:------ 13:042805 3EF0 003 0000FF
+        # .I --- 13:023770 --:------ 13:023770 3EF0 003 00C8FF
+        assert payload[2:4] in ("00", "C8"), f"byte 1: {payload[2:4]}"
         assert payload[4:6] == "FF", f"byte 2: {payload[4:6]}"
 
-    if msg.len > 3:  # for all OTB
-        if payload[2:4] != "FF":
-            assert int(payload[2:4], 16) <= 100, f"byte 1: {payload[2:4]}"
+    if msg.len >= 6:  # RP|OTB|006 (to RQ|CTL/HGI/RFG)
+        # RP --- 10:105624 01:133689 --:------ 3EF0 006 0000100000FF
+        # RP --- 10:105624 01:133689 --:------ 3EF0 006 003B100C00FF
+        assert int(payload[2:4], 16) <= 100, f"byte 1: {payload[2:4]}"
         assert payload[4:6] in ("10", "11"), f"byte 2: {payload[4:6]}"
+
+    result = {
+        "modulation_level": percent(payload[2:4]),  # TODO: rel_modulation_level
+        "_flags_0": payload[4:6],
+    }
+
+    if msg.len >= 6:  # RP|OTB|006 (to RQ|CTL/HGI/RFG)
         # RP --- 10:138822 01:187666 --:------ 3EF0 006 000110FA00FF  # ?corrupt
+
+        # for OTB (there's no reliable) modulation_level <-> flame_state)
+        assert payload[6:8] in (
+            "00",
+            "01",
+            "02",
+            "04",
+            "08",
+            "0A",
+            "0C",
+            "42",
+        ), f"byte 3: {payload[6:8]}"
         assert int(payload[6:8], 16) & 0b11110000 == 0, f"byte 3: {payload[6:8]}"
         assert payload[8:10] in (
             "00",
@@ -1771,24 +1686,9 @@ def parser_3ef0(payload, msg) -> dict:
         ), f"byte 4: {payload[8:10]}"
         assert payload[10:12] in ("00", "1C", "FF"), f"byte 5: {payload[10:12]}"
 
-    if msg.len > 6:  # <= 9: # for some OTB
-        assert int(payload[12:14], 16) & 0b11111100 == 0, f"byte 6: {payload[12:14]}"
-        assert payload[-2:] in ("00", "64"), f"byte x: {payload[-2:]}"
-
-    result = {
-        "actuator_enabled": bool(percent(payload[2:4])),
-        "modulation_level": percent(payload[2:4]),  # TODO: rel_modulation_level
-        "_unknown_0": payload[4:6],
-    }
-
-    if msg.len > 3:  # for OTB (there's no reliable) modulation_level <-> flame_state)
-        # assert payload[6:8] in (
-        #     "00", "01", "02", "04", "08", "0A", "0C", "42",
-        # ), payload[6:8]
-
         result.update(
             {
-                "_unknown_3": flag8(payload[6:8]),
+                "_flags_3": flag8(payload[6:8]),
                 "ch_enabled": bool(int(payload[6:8], 0x10) & 1 << 1),
                 "dhw_active": bool(int(payload[6:8], 0x10) & 1 << 2),
                 "flame_active": bool(int(payload[6:8], 0x10) & 1 << 3),
@@ -1797,10 +1697,15 @@ def parser_3ef0(payload, msg) -> dict:
             }
         )
 
-    if msg.len > 6:
+    if msg.len >= 9:  # I/RP|OTB|009
+        assert int(payload[12:14], 16) & 0b11111100 == 0, f"byte 6: {payload[12:14]}"
+        assert int(payload[12:14], 16) & 0b00000010 == 2, f"byte 6: {payload[12:14]}"
+        assert 10 <= int(payload[14:16], 16) <= 80, f"byte 7: {payload[14:16]}"
+        assert int(payload[16:18], 16) in (0, 100), f"byte 8: {payload[18:]}"
+
         result.update(
             {
-                "_unknown_6": flag8(payload[12:14]),
+                "_flags_6": flag8(payload[12:14]),
                 "ch_active": bool(int(payload[12:14], 0x10) & 1 << 0),
                 "ch_setpoint": int(payload[14:16], 0x10),
                 "max_rel_modulation": int(payload[16:18], 0x10),
@@ -1827,20 +1732,31 @@ def parser_3ef1(payload, msg) -> dict:
             "blob": payload[8:],
         }
 
-    assert msg.verb == RP, f"verb should be {RP}, not: {msg.verb}"
-    assert msg.len == 7, msg.len
-    assert payload[:2] == "00", payload[:2]
-    assert percent(payload[10:12]) <= 1, payload[10:12]
-    # assert payload[12:] == "FF"
+    if payload[12:] == "FF":  # is BDR
+        # assert (
+        #     re.compile(r"^00[0-9A-F]{10}FF").match(payload)
+        # ), "doesn't match: " + r"^00[0-9A-F]{10}FF"
+        assert int(payload[2:6], 16) <= 7200, int(payload[2:6], 16)
+        assert payload[6:10] in ("87B3", "9DFA", "DCE1", "E638", "F8F7") or (
+            int(payload[6:10], 16) <= 7200
+        ), int(payload[6:10], 16)
+        assert percent(payload[10:12]) in (0, 1), percent(payload[10:12])
+
+    else:  # is OTB?
+        # assert (
+        #     re.compile(r"^00[0-9A-F]{10}10").match(payload)
+        # ), "doesn't match: " + r"^00[0-9A-F]{10}10"
+        assert payload[2:6] == "7FFF", payload[2:6]
+        assert payload[6:10] == "003C", payload[6:10]  # 1 minute
+        assert percent(payload[10:12]) <= 1, payload[10:12]
 
     cycle_countdown = None if payload[2:6] == "7FFF" else int(payload[2:6], 16)
 
     return {
-        "actuator_enabled": bool(percent(payload[10:12])),
         "modulation_level": percent(payload[10:12]),
         "actuator_countdown": int(payload[6:10], 16),
-        "cycle_countdown": cycle_countdown,  # not for OTB, == "7FFF"
-        "_unknown_0": payload[12:],  # for OTB != "FF"
+        "cycle_countdown": cycle_countdown,
+        "_unknown_0": payload[12:],
     }
 
 
@@ -1886,32 +1802,3 @@ PAYLOAD_PARSERS = {
     for k, v in locals().items()
     if callable(v) and k.startswith("parser_") and k != "parser_unknown"
 }
-
-
-def parse_payload(msg, logger=_LOGGER) -> dict:
-
-    parser = PAYLOAD_PARSERS.get(msg.code, parser_unknown)
-    result = None
-
-    try:  # parse the packet
-        result = parser(msg.raw_payload, msg)
-        assert isinstance(result, (dict, list)), f"invalid payload type: {type(result)}"
-
-    except AssertionError as err:
-        # beware: HGI80 can send parseable but 'odd' packets +/- get invalid reply
-        (logger.exception if DEV_MODE and msg.src.type != "18" else logger.warning)(
-            "%s << %s", msg._pkt, f"{err.__class__.__name__}({err})"
-        )
-
-    except (InvalidPacketError, InvalidPayloadError) as err:  # CorruptEvohomeError
-        (logger.exception if DEV_MODE else logger.warning)("%s << %s", msg._pkt, err)
-
-    except (AttributeError, LookupError, TypeError, ValueError) as err:  # TODO: dev
-        logger.exception(
-            "%s << Coding error: %s", msg._pkt, f"{err.__class__.__name__}({err})"
-        )
-
-    except NotImplementedError:  # parser_unknown (unknown packet code)
-        logger.warning("%s << Unknown packet code (cannot parse)", msg._pkt)
-
-    return result
