@@ -10,14 +10,14 @@ import asyncio
 import logging
 from datetime import datetime as dt
 from datetime import timedelta as td
-from queue import Empty, PriorityQueue, SimpleQueue
+from queue import Empty, Full, PriorityQueue, SimpleQueue
 from typing import Callable, List, Optional, Tuple
 
 from .command import ARGS, DEAMON, EXPIRES, FUNC, TIMEOUT, Command
 from .const import __dev_mode__
 from .message import Message
 
-DONT_CREATE_MESSAGES = 2  # duplicate
+DONT_CREATE_MESSAGES = 3  # duplicate
 
 DEV_MODE = __dev_mode__ and False
 
@@ -125,12 +125,15 @@ class MessageTransport(asyncio.Transport):
                         await call_send_data(cmd)
                 except (AssertionError, NotImplementedError):  # TODO: needs checking
                     pass
+                # except:
+                #     _LOGGER.exception("")
+                #     continue
 
                 self._que.task_done()
                 # if self._write_buffer_paused:
                 self.get_write_buffer_size()
 
-            _LOGGER.debug("MsgTransport.pkt_dispatcher(): connection_lost(None)")
+            _LOGGER.error("MsgTransport.pkt_dispatcher(): connection_lost(None)")
             [p.connection_lost(None) for p in self._protocols]
 
         self._dispatcher = dispatcher
@@ -174,15 +177,8 @@ class MessageTransport(asyncio.Transport):
         if self._gwy.config.reduce_processing >= DONT_CREATE_MESSAGES:
             return
 
-        try:
-            msg = Message(self._gwy, pkt)  # trap/logs all invalid msgs appropriately
+        msg = Message(self._gwy, pkt)  # trap/logs all invalid msgs appropriately
 
-        except (AssertionError, ValueError) as exc:
-            if DEV_MODE:
-                _LOGGER.error("%s < Cant create message (ignoring): %s", pkt, exc)
-            return
-
-        # _LOGGER.info("MsgTransport._pkt_receiver(pkt): %s", msg)
         # NOTE: msg._pkt._hdr is expensive - don't call it unless there's callbacks
         if self._callbacks and msg._pkt._hdr in self._callbacks:
             callback = self._callbacks[msg._pkt._hdr]  # 3rd, invoke any callback
@@ -194,7 +190,7 @@ class MessageTransport(asyncio.Transport):
             try:
                 p.data_received(msg)
             except:  # noqa: E722  # TODO: remove broad-except
-                pass
+                _LOGGER.exception("Exception in callback to application layer")
 
     def close(self):
         """Close the transport.
@@ -340,7 +336,10 @@ class MessageTransport(asyncio.Transport):
             if not self._dispatcher:  # TODO: do better?
                 _LOGGER.warning("MsgTransport.write(%s): no dispatcher", cmd)
 
-            self._que.put_nowait(cmd)  # was: self._que.put_nowait(cmd)
+            try:
+                self._que.put_nowait(cmd)
+            except Full:
+                pass  # TODO: why? - consider restarting the dispatcher
 
         # self.get_write_buffer_size()  # TODO: how to auto unpause?
 
