@@ -124,9 +124,12 @@ RAMSES_CODES = {  # rf_unknown
         RQ: r"^00$",
         RP: r"^00[0-9A-F]{2}$",  # seems only 13: RP (TODO: what about 10:, 08/31:)
     },
-    _0009: {  # relay_failsafe
+    _0009: {  # relay_failsafe (only is_controller, OTB send an 0009?)
         NAME: "relay_failsafe",
+        #  I --- 01:145038 --:------ 01:145038 0009 006 FC01FFF901FF
+        #  I --- 01:145038 --:------ 01:145038 0009 003 0700FF
         #  I --- 10:040239 01:223036 --:------ 0009 003 000000
+        #  I --- --:------ --:------ 12:227486 0009 003 0000FF
         I_: r"^((0[0-9A-F]|F[9AC])0[0-1](00|FF))+$",
     },
     _000A: {  # zone_params
@@ -187,7 +190,7 @@ RAMSES_CODES = {  # rf_unknown
         RQ: r"^0000[0-3][0-9A-F]$",  # f"0000{log_idx}", no payload
     },
     _042F: {  # unknown, # non-evohome are len==9, seen only once?
-        # 16:48:11.813119 060  I --- 32:168090 --:------ 32:168090 042F 009 000000100F00105050  # noqa
+        # .I --- 32:168090 --:------ 32:168090 042F 009 000000100F00105050
         NAME: "message_042f",
         I_: r"^00([0-9A-F]{2}){7}$",
     },
@@ -343,26 +346,26 @@ RAMSES_CODES = {  # rf_unknown
     },  # minutes
     _2309: {  # setpoint
         NAME: "setpoint",
-        RQ: r"^0[0-9A-F]([0-9A-F]{4})?$",  # NOTE: 12 uses: r"^0[0-9A-F]$"
         I_: r"^(0[0-9A-F]{5})+$",
         # RQ --- 12:010740 01:145038 --:------ 2309 003 03073A # No RPs
+        RQ: r"^0[0-9A-F]([0-9A-F]{4})?$",  # NOTE: 12 uses: r"^0[0-9A-F]$"
         EXPIRES: td(minutes=30),
     },
     _2349: {  # zone_mode
         NAME: "zone_mode",
-        # RQ: r"^0[0-9A-F](00)?$",  # is usually: r"^0[0-9A-F]$"
-        RQ: r"^0[0-9A-F].*",  # r"^0[0-9A-F](00)?$",
-        I_: r"^0[0-9A-F](([0-9A-F]){12}){1,2}$",
+        I_: r"^0[0-9A-F]([0-9A-F]{12}){1,2}$",
+        # RQ --- 22:070483 01:063844 --:------ 2349 007 06070803000027
+        RQ: r"^0[0-9A-F](00|[0-9A-F]{12})?$",
         EXPIRES: td(hours=4),
     },
     _2D49: {  # unknown
         NAME: "message_2d49",
-        I_: r"^(0[0-9A-F]|88|FD)[0-9A-F]{2}00$",
         # 10:14:08.526 045  I --- 01:023389 --:------ 01:023389 2D49 003 010000
         # 10:14:12.253 047  I --- 01:023389 --:------ 01:023389 2D49 003 00C800
         # 10:14:12.272 047  I --- 01:023389 --:------ 01:023389 2D49 003 01C800
         # 10:14:12.390 049  I --- 01:023389 --:------ 01:023389 2D49 003 880000
         # 10:14:12.399 048  I --- 01:023389 --:------ 01:023389 2D49 003 FD0000
+        I_: r"^(0[0-9A-F]|88|FD)[0-9A-F]{2}00$",
     },  # seen with Hometronic systems
     _2E04: {  # system_mode
         NAME: "system_mode",
@@ -417,25 +420,33 @@ RAMSES_CODES = {  # rf_unknown
         RQ: r"^00[0-9A-F]{4}0{4}$",  # is strictly: r"^00[0-9A-F]{8}$",
         RP: r"^00[0-9A-F]{8}$",
     },
-    _3B00: {  # actuator_sync
+    _3B00: {  # actuator_sync, NOTE: no RQ
         NAME: "actuator_sync",
         I_: r"^(00|FC)(00|C8)$",
-    },  # No RQ
+    },
     _3EF0: {  # actuator_state
         NAME: "actuator_state",
-        # I_: r"^00[0-9A-C][0-9A-F]([0-9A-F]{6})?FF$",
-        I_: r"^00",
-        RQ: r"^00$",
-        RP: r"^00",
+        # .I --- 13:106039 --:------ 13:106039 3EF0 003 00C8FF
+        # .I --- 10:030051 --:------ 10:030051 3EF0 009 000010000000020A64
+        # .I --- 08:031043 31:077159 --:------ 3EF0 020 001191A72044399D2A50DE43F920478AF7185F3F  # Jasper
+        I_: r"^00((00|C8)FF|[0-9A-F]{16}|[0-9A-F]{38})$",  # NOTE: latter is Japser
+        RQ: r"^00(00)?$",
+        RP: r"^00((00|C8)FF|[0-9A-F]{10}|[0-9A-F]{16})$",
     },
     _3EF1: {  # actuator_cycle
         NAME: "actuator_cycle",
-        RQ: r"^(0[0-9A-F](00)?|00[0-9A-F]{22})$",  # NOTE: both seen in the wild
-        RP: r"^(0[0-9A-F]{13}|00[0-9A-F]{22})$",  # TODO
+        # RQ --- 31:004811 13:077615 --:------ 3EF1 001 00
+        # RP --- 13:077615 31:004811 --:------ 3EF1 007 00024D001300FF
+        # RQ --- 22:068154 13:031208 --:------ 3EF1 002 0000
+        # RP --- 13:031208 22:068154 --:------ 3EF1 007 00024E00E000FF
+        # RQ --- 31:074182 08:026984 --:------ 3EF1 012 0005D1341DA39B8C7DAFD4C1
+        # RP --- 08:026984 31:074182 --:------ 3EF1 018 001396A7E087922FA77794280B66BE16A975
+        RQ: r"^00((00)?|[0-9A-F]{22})$",  # NOTE: latter is Japser
+        RP: r"^00([0-9A-F]{12}|[0-9A-F]{34})$",  # NOTE: latter is Japser
     },
     _PUZZ: {
         NAME: "puzzle_packet",
-        I_: r"^00[0-9A-F]{12}7F[0-9A-F]{4}7F[0-9A-F]{4}(7F)+",
+        I_: r"^00([0-9A-F]+7F)+",
     },
 }
 for code in RAMSES_CODES.values():
@@ -679,8 +690,7 @@ RAMSES_DEVICES = {
         _3EF1: {RQ: {}},
     },
     "13": {  # e.g. BDR91A/BDR91T: Wireless Relay Box
-        _0008: {RP: {}},
-        # _0009: {RP: {}},  # TODO: needs confirming
+        _0008: {RP: {}},  # doesn't RP/0009
         _0016: {RP: {}},
         # _10E0: {},  # 13: will not RP/10E0 # TODO: how to indicate that fact here
         _1100: {I_: {}, RP: {}},
