@@ -341,7 +341,7 @@ class PacketProtocolBase(asyncio.Protocol):
         else:
             self._exclude = list(gwy._exclude.keys())
             self._include = []
-        self._unwanted = []
+        self._unwanted = [NON_DEVICE_ID, NUL_DEVICE_ID]
 
         self._hgi80 = {
             IS_INITIALIZED: None,
@@ -376,30 +376,36 @@ class PacketProtocolBase(asyncio.Protocol):
         An unwanted device_id will 'trump' a whitelited device_id in the same packet.
         """
 
-        for dev_id in [
-            d
-            for d in dict.fromkeys((src_id, dst_id))
-            if d not in (NON_DEVICE_ID, NUL_DEVICE_ID)
-        ]:
-            if dev_id in self._unwanted:  # used to avoid duplicate logging
-                return
-
-            if self._include and dev_id not in self._include:
-                _LOGGER.warning(
-                    f"Ignoring a non-allowed device_id: {dev_id}, "
-                    f"if required, add it to the {KNOWN_LIST}"
-                )
-                self._unwanted.append(dev_id)
+        for dev_id in [d for d in dict.fromkeys((src_id, dst_id))]:
+            if dev_id in self._unwanted:
                 return
 
             if dev_id in self._exclude:
-                _LOGGER.warning(
-                    f"Ignoring a blocked device_id: {dev_id}, "
+                _LOGGER.info(
+                    f"Blocking packets with device_id: {dev_id} (is blacklisted), "
                     f"if required, remove it from the {BLOCK_LIST})"
                 )
                 self._unwanted.append(dev_id)
                 return
 
+            if not self._include or dev_id in self._include:
+                break
+
+            if dev_id == self._hgi80[DEVICE_ID]:
+                _LOGGER.warning(
+                    f"Allowing packets with device_id: {dev_id} (is active gateway?), "
+                    f"configure the {KNOWN_LIST}/{BLOCK_LIST} as required"
+                )
+                self._include.append(dev_id)  # the only time include list is modified
+                break
+
+            _LOGGER.warning(
+                f"Blocking packets with device_id: {dev_id} (is not whitelisted), "
+                f"if required, add it to the {KNOWN_LIST}"
+            )
+            self._unwanted.append(dev_id)
+            return
+
         return True
 
     def _pkt_received(self, pkt: Packet) -> None:
@@ -422,46 +428,31 @@ class PacketProtocolBase(asyncio.Protocol):
         try:
             pkt = Packet.from_port(self._gwy, dtm, line, raw_line=raw_line)
 
-        except (AssertionError, LookupError, TypeError, ValueError) as exc:
-            if DEV_MODE and line and line[:1] != "#" and "*" not in line:
-                _LOGGER.exception("%s < Cant create packet (ignoring 1): %s", line, exc)
-            return
-
         except InvalidPacketError as exc:
             if "# evofw" in line and self._hgi80[IS_EVOFW3] is None:
-                self._hgi80[IS_EVOFW3] = True
+                self._hgi80[IS_EVOFW3] = line
                 if self._evofw_flag not in (None, "!V"):
                     self._transport.write(
                         bytes(f"{self._evofw_flag}\r\n".encode("ascii"))
                     )
-            elif was_initialized:
-                _LOGGER.exception("%s < Cant create packet (ignoring 2): %s", line, exc)
+            elif was_initialized and line and line[:1] != "#" and "*" not in line:
+                _LOGGER.error("%s < Cant create packet (ignoring): %s", line, exc)
             return
 
         if pkt.src.type == "18":
             if self._hgi80[DEVICE_ID] is None:
                 self._hgi80[DEVICE_ID] = pkt.src.id
-                if self._include and pkt.src.id not in self._include:
-                    self._include.append(pkt.src.id)  # add to whitelist
 
-            elif (
-                self._hgi80[DEVICE_ID] != pkt.src.id
-                and pkt.src.id not in self._unwanted
-            ):
-                _LOGGER.error(
-                    f"{pkt} < seems to be more than one HGI80-compatible device"
-                    f" (another is: {self._hgi80[DEVICE_ID]}), this is unsupported"
+            elif self._hgi80[DEVICE_ID] != pkt.src.id:
+                (_LOGGER.debug if pkt.src.id in self._unwanted else _LOGGER.error)(
+                    f"{pkt} < There appears to be more than one HGI80-compatible device"
+                    f" (active gateway: {self._hgi80[DEVICE_ID]}), this is unsupported"
                 )
-                _LOGGER.warning(
-                    f"Ignoring a secondary gateway: {pkt.src.id}, "
-                    f"configure the {KNOWN_LIST}/{BLOCK_LIST} as required"
-                )
-                self._unwanted.append(pkt.src.id)
 
         try:
             self._pkt_received(pkt)
         except:  # noqa: E722  # TODO: remove broad-except
-            pass
+            _LOGGER.exception("Exception in callback to message layer")
 
     def data_received(self, data: ByteString) -> None:
         """Called when some data (packet fragments) is received (from RF)."""
@@ -480,18 +471,15 @@ class PacketProtocolBase(asyncio.Protocol):
         for dtm, raw_line in _bytes_received(data):
             self._line_received(dtm, _normalise(_str(raw_line)), raw_line)
 
-    async def _send_data(self, data: str, ignore_pause=False) -> None:
-        """Send a bytearray to the transport (serial) interface.
+    async def _send_data(self, data: str) -> None:
+        """Send a bytearray to the transport (serial) interface."""
 
-        The _pause_writing flag can be ignored, is useful for sending traceflags.
-        """
+        while self._pause_writing:
+            await asyncio.sleep(0.005)
 
-        if not ignore_pause:
-            while self._pause_writing:
-                await asyncio.sleep(0.005)
         while (
             self._transport is None
-            or self._transport.serial is None  # Shouldn't be required, but is!
+            # or self._transport.serial is None  # Shouldn't be required, but is!
             or getattr(self._transport.serial, "out_waiting", False)
         ):
             await asyncio.sleep(0.005)
@@ -516,9 +504,19 @@ class PacketProtocolBase(asyncio.Protocol):
         #     _LOGGER.warning(
         #     return
 
-        if cmd.src.type != "18":
-            _LOGGER.info("PktProtocol.send_data(%s): IMPERSONATING!", cmd.tx_header)
-            await self._send_data(str(Command._puzzle("02", cmd.tx_header)))
+        if cmd.src.id != HGI_DEVICE_ID:
+            if self._hgi80[IS_EVOFW3]:
+                _LOGGER.info(
+                    "Impersonating device: %s, for pkt: %s", cmd.src.id, cmd.tx_header
+                )
+            else:
+                _LOGGER.warning(
+                    "Impersonating device: %s, for pkt: %s"
+                    ", NB: standard HGI80s dont support this feature, it needs evofw3!",
+                    cmd.src.id,
+                    cmd.tx_header,
+                )
+            await self.send_data(Command._puzzle("02", cmd.tx_header))
 
         await self._send_data(str(cmd))
 
@@ -544,13 +542,6 @@ class PacketProtocolBase(asyncio.Protocol):
 class PacketProtocolPort(PacketProtocolBase):
     """Interface for a packet protocol (without QoS)."""
 
-    def __init__(self, gwy, pkt_handler: Callable) -> None:
-        super().__init__(gwy, pkt_handler)
-
-        # this needs to be here, and not in connection_made, so that it is 1st
-        if not self._disable_sending:
-            self._loop.create_task(self.send_data(INIT_CMD))  # HACK: port wakeup
-
     def connection_made(self, transport: asyncio.Transport) -> None:
         """Called when a connection is made."""
         _LOGGER.info(f"RAMSES_RF protocol library v{VERSION} (serial port)")
@@ -558,8 +549,13 @@ class PacketProtocolPort(PacketProtocolBase):
         super().connection_made(transport)  # self._transport = transport
         # self._transport.serial.rts = False
 
-        # Used to see if using a evofw3 rather than a HGI80
+        # determine if using a evofw3 rather than a HGI80
         self._transport.write(bytes("!V\r\n".encode("ascii")))
+
+        # add this to start of the pkt log, if any
+        if not self._disable_sending:
+            self._loop.create_task(self.send_data(INIT_CMD))
+
         self.resume_writing()
 
 
@@ -592,28 +588,15 @@ class PacketProtocolRead(PacketProtocolBase):
 
     def _line_received(self, dtm: str, line: str, raw_line: str) -> None:
 
-        self._hgi80[IS_INITIALIZED] = True
-
         try:
             pkt = Packet.from_file(self._gwy, dtm, line)
-
-        except (AssertionError, LookupError, TypeError, ValueError) as exc:
-            if (
-                line
-                and line[:1] != "#"
-                and "*" not in line
-                and (dtm and dtm.lstrip()[:1] != "#")
-            ):
-                _LOGGER.error("%s < Cant create packet (ignoring): %s", line, exc)
-            return
-
-        except InvalidPacketError:
+        except (InvalidPacketError, ValueError):  # VE due to dt.fromisoformat()
             return
 
         try:
             self._pkt_received(pkt)
         except:  # noqa: E722  # TODO: remove broad-except
-            pass
+            _LOGGER.exception("Exception in callback to message layer")
 
     def data_received(self, data: str) -> None:
         """Called when a packet line is received (from a log file)."""
@@ -768,6 +751,8 @@ class PacketProtocolQos(PacketProtocolPort):
         while self._qos_cmd is not None:
             await asyncio.sleep(0.005)
 
+        await super().send_data(cmd)
+
         self._qos_lock.acquire()
         self._qos_cmd = cmd
         self._qos_lock.release()
@@ -776,7 +761,6 @@ class PacketProtocolQos(PacketProtocolPort):
         self._tx_retries = 0
         self._tx_retry_limit = cmd.qos.get("retries", QOS_TX_RETRIES)
 
-        await super().send_data(cmd)
         self._timeouts(dt.now())
 
         while self._qos_cmd is not None:  # until sent (may need re-transmit) or expired
@@ -839,9 +823,9 @@ def create_pkt_stack(
     """
 
     def _protocol_factory():
-        if packet_dict or packet_log:
+        if packet_log or packet_dict is not None:
             return create_protocol_factory(PacketProtocolRead, gwy, pkt_callback)()
-        elif gwy.config.disable_sending:
+        elif gwy.config.disable_sending:  # TODO: assumes we wont change our mind
             return create_protocol_factory(PacketProtocol, gwy, pkt_callback)()
         else:
             return create_protocol_factory(PacketProtocolQos, gwy, pkt_callback)()
