@@ -14,7 +14,7 @@ from typing import List, Optional
 
 from .const import DISCOVER_ALL, DISCOVER_PARAMS, DISCOVER_SCHEMA, DISCOVER_STATUS
 from .devices import BdrSwitch, Device, OtbGateway
-from .entities import Entity, discover_decorator
+from .entities import Entity, discovery_filter
 from .protocol import Command, FaultLog, Priority
 from .protocol.const import (
     _000C_DEVICE,
@@ -124,7 +124,7 @@ class SysFaultLog:  # 0418
         super().__init__(*args, **kwargs)
         self._fault_log = FaultLog(self._ctl)
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
@@ -152,7 +152,7 @@ class SysFaultLog:  # 0418
 
 
 class SysDatetime:  # 313F
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
@@ -179,7 +179,7 @@ class SysDatetime:  # 313F
 
 
 class SysLanguage:  # 0100
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
@@ -198,7 +198,7 @@ class SysLanguage:  # 0100
 
 
 class SysMode:  # 2E04
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
@@ -375,7 +375,7 @@ class MultiZone:  # 0005 (+/- 000C?)
 
         self._prev_30c9 = None  # used to eavesdrop zone sensors
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
@@ -683,7 +683,7 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
     # def __str__(self) -> str:  # TODO: WIP
     #     return json.dumps({self._ctl.id: self.schema})
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # super()._discover(discover_flag=discover_flag)
 
@@ -940,11 +940,13 @@ class System(StoredHw, SysDatetime, SysFaultLog, SystemBase):
 
     @property
     def heat_demands(self) -> Optional[dict]:  # 3150
+        # FC: 00-C8 (no F9, FA), TODO: deprecate as FC only?
         if self._heat_demands:
             return {k: v.payload["heat_demand"] for k, v in self._heat_demands.items()}
 
     @property
     def relay_demands(self) -> Optional[dict]:  # 0008
+        # FC: 00-C8, F9: 00-C8, FA: 00 or C8 only (01: all 3, 02: FC/FA only)
         if self._relay_demands:
             return {
                 k: v.payload["relay_demand"] for k, v in self._relay_demands.items()
@@ -953,7 +955,7 @@ class System(StoredHw, SysDatetime, SysFaultLog, SystemBase):
     @property
     def relay_failsafes(self) -> Optional[dict]:  # 0009
         if self._relay_failsafes:
-            return {}  # failsafe_enabled
+            return {}  # TODO: failsafe_enabled
 
     @property
     def status(self) -> dict:
@@ -979,7 +981,7 @@ class Evohome(SysLanguage, SysMode, MultiZone, UfhSystem, System):  # evohome
     def __repr__(self) -> str:
         return f"{self._ctl.id} (evohome)"
 
-    # @discover_decorator  # HACK: remove me
+    # @discovery_filter  # HACK: remove me
     # def _discover(self, discover_flag=DISCOVER_ALL) -> None:
     #     super()._discover(discover_flag=discover_flag)
 
@@ -1024,6 +1026,8 @@ class Hometronics(System):
     #  I --- 01:023389 --:------ 01:023389 2D49 003 880000
     #  I --- 01:023389 --:------ 01:023389 2D49 003 FD0000
 
+    # Hometronic does not react to W/2349 but rather requies W/2309
+
     _PROFILE = SYSTEM_PROFILE.EVO
 
     RQ_SUPPORTED = (_0004, _000C, _2E04, _313F)  # TODO: WIP
@@ -1032,7 +1036,7 @@ class Hometronics(System):
     def __repr__(self) -> str:
         return f"{self._ctl.id} (hometronics)"
 
-    # @discover_decorator
+    # @discovery_filter
     # def _discover(self, discover_flag=DISCOVER_ALL) -> None:
     #     # super()._discover(discover_flag=discover_flag)
 
