@@ -14,7 +14,7 @@ from typing import Optional
 
 from .const import DISCOVER_ALL, DISCOVER_PARAMS, DISCOVER_SCHEMA, DISCOVER_STATUS
 from .devices import BdrSwitch, Device, DhwSensor
-from .entities import Entity, discover_decorator
+from .entities import Entity, discovery_filter
 from .protocol import Command, Schedule
 from .protocol.const import (
     _000C_DEVICE,
@@ -184,7 +184,7 @@ class ZoneSchedule:  # 0404  # TODO: add for DHW
 
         self._schedule = Schedule(self)
 
-    # @discover_decorator
+    # @discovery_filter
     # def _discover(self, discover_flag=DISCOVER_ALL) -> None:
     #     if discover_flag & DISCOVER_STATUS:  # TODO: add back in
     #         self._loop.create_task(self.get_schedule())  # 0404
@@ -219,7 +219,7 @@ class ZoneSchedule:  # 0404  # TODO: add for DHW
 class RelayDemand:  # 0008
     """Not all zones call for heat."""
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
@@ -267,7 +267,7 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         if htg_valve:
             self._set_htg_valve(htg_valve)
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # super()._discover(discover_flag=discover_flag)
 
@@ -367,11 +367,11 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         return self._msg_value(_1F41)
 
     @property
-    def setpoint(self) -> Optional[float]:  # 1F41
-        return self._msg_value(_1F41, key=ATTR_SETPOINT)
+    def setpoint(self) -> Optional[float]:  # 10A0
+        return self._msg_value(_10A0, key=ATTR_SETPOINT)
 
     @setpoint.setter
-    def setpoint(self, value) -> None:  # 1F41
+    def setpoint(self, value) -> None:  # 10A0
         return self.set_config(setpoint=value)
 
     @property
@@ -392,8 +392,9 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
 
     def set_mode(self, mode=None, active=None, until=None) -> Task:
         """Set the DHW mode (mode, active, until)."""
-        cmd = Command.set_dhw_mode(self._ctl.id, mode=mode, active=active, until=until)
-        return self._gwy.send_cmd(cmd)
+        return self._gwy.send_cmd(
+            Command.set_dhw_mode(self._ctl.id, mode=mode, active=active, until=until)
+        )
 
     def set_boost_mode(self) -> Task:
         """Enable DHW for an hour, despite any schedule."""
@@ -415,8 +416,9 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         # if differential is None:
         #     setpoint = dhw_params["differential"]
 
-        cmd = Command.set_dhw_params(self._ctl.id, setpoint, overrun, differential)
-        return self._gwy.send_cmd(cmd)
+        return self._gwy.send_cmd(
+            Command.set_dhw_params(self._ctl.id, setpoint, overrun, differential)
+        )
 
     def reset_config(self) -> Task:  # 10A0
         """Reset the DHW parameters to their default values."""
@@ -470,7 +472,7 @@ class Zone(ZoneSchedule, ZoneBase):
         if sensor:
             self._set_sensor(sensor)
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
@@ -642,15 +644,12 @@ class Zone(ZoneSchedule, ZoneBase):
         return self._msg_value((_2309, _2349), key=ATTR_SETPOINT)
 
     @setpoint.setter
-    def setpoint(self, value) -> None:
+    def setpoint(self, value) -> None:  # 000A/2309
         """Set the target temperature, until the next scheduled setpoint."""
         if value is None:
             self.reset_mode()
         else:
-            cmd = Command.set_zone_setpoint(self._ctl.id, self.idx, value)
-            self._gwy.send_cmd(cmd)
-            # NOTE: the following doesn't wotk for e.g. Hometronics
-            # self.set_mode(mode=ZONE_MODE.advanced_override, setpoint=value)
+            self._gwy.send_cmd(Command.set_zone_setpoint(self._ctl.id, self.idx, value))
 
     @property
     def temperature(self) -> Optional[float]:  # 30C9
@@ -677,7 +676,11 @@ class Zone(ZoneSchedule, ZoneBase):
         ]
         return any(windows) if windows else None
 
-    def reset_config(self) -> Task:
+    def _get_temp(self) -> Task:  # TODO: messy - needs tidy up
+        """Get the zone's latest temp from the Controller."""
+        return self._gwy.send_cmd(Command.get_zone_temp(self._ctl.id, self.idx))
+
+    def reset_config(self) -> Task:  # 000A
         """Reset the zone's parameters to their default values."""
         return self.set_config()
 
@@ -709,15 +712,17 @@ class Zone(ZoneSchedule, ZoneBase):
         """Set the zone to the lowest possible setpoint, indefinitely."""
         return self.set_mode(mode=ZONE_MODE.permanent_override, setpoint=5)  # TODO
 
-    def set_mode(self, mode=None, setpoint=None, until=None) -> Task:
+    def set_mode(self, mode=None, setpoint=None, until=None) -> Task:  # 2309/2349
         """Override the zone's setpoint for a specified duration, or indefinitely."""
-        cmd = Command.set_zone_mode(self._ctl.id, self.idx, mode, setpoint, until)
+        if mode is None and until is None:  # Hometronics doesn't support 2349
+            cmd = Command.set_zone_setpoint(self._ctl.id, self.idx, setpoint)
+        else:
+            cmd = Command.set_zone_mode(self._ctl.id, self.idx, mode, setpoint, until)
         return self._gwy.send_cmd(cmd)
 
     def set_name(self, name) -> Task:
         """Set the zone's name."""
-        cmd = Command.set_zone_name(self._ctl.id, self.idx, name)
-        return self._gwy.send_cmd(cmd)
+        return self._gwy.send_cmd(Command.set_zone_name(self._ctl.id, self.idx, name))
 
     @property
     def schema(self) -> dict:
@@ -760,7 +765,7 @@ class EleZone(RelayDemand, Zone):  # BDR91A/T  # TODO: 0008/0009/3150
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
@@ -793,7 +798,7 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
@@ -823,7 +828,7 @@ class RadZone(Zone):  # HR92/HR80
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
@@ -839,7 +844,7 @@ class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
@@ -866,7 +871,7 @@ class ValZone(EleZone):  # BDR91A/T
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discover_decorator
+    @discovery_filter
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
